//
// "$Id: browser2.cxx,v 1.5.2.6.2.7 2004/07/26 20:52:52 easysw Exp $"
//
// Browser test program for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2004 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "fltk-bugs@fltk.org".
//

#warning Currently not rendering embeded colors. -rs

/*
This is a test of how the browser draws lines.
This is a second line.
This is a third.

That was a blank line above this.

@r@_Right justify
@c@_Center justify
@_Left justify

@bBold text
@iItalic text
@b@iBold Italic
@fFixed width
@f@bBold Fixed
@f@iItalic Fixed
@f@i@bBold Italic Fixed
@lLarge
@l@bLarge bold
@sSmall
@s@bSmall bold
@s@iSmall italic
@s@i@bSmall italic bold
@uunderscore
@C1RED
@C2Green
@C4Blue

  You should try different browser types:
  Fl_Browser
  Fl_Select_Browser
  Fl_Hold_Browser
  Fl_Multi_Browser
*/

#include <fltk/run.h>
#include <fltk/Color.h>
#include <fltk/Style.h>
#include <fltk/FL_API.h>
#include <fltk/error.h>
#include <sys/types.h>
#include <fltk/events.h>
//#include <FL/Fl_Select_Browser.H>
#include <fltk/Browser.h>
#include <fltk/FileBrowser.h> 
#include <fltk/InputBrowser.h> 
#include <fltk/MultiBrowser.h>

#include <fltk/DoubleBufferWindow.h>
#include <fltk/Button.h>
#include <fltk/IntInput.h>
#include <fltk/Browser.h>
#include <fltk/ask.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h> // chdir()

fltk::Browser *browser;
fltk::Button  *top,
    *bottom,
    *middle,
    *visible,
    *swap;
    fltk::IntInput  *field;


void b_cb(fltk::Widget* o, void*) 
{
  printf("callback, selection = %d, event_clicks = %d\n",
   ((fltk::Browser*)o)->value(), fltk::event_clicks());
}

void show_cb(fltk::Widget *o, void *) 
{
  int line = atoi(field->text());

  if (!line) 
  {
    fltk::alert("Please enter a number in the text field\n"
        "before clicking on the buttons.");
    return;
  }

  if (o == top)
    browser->topline(line);
  else if (o == bottom)
    browser->bottomline(line);
  else if (o == middle)
    browser->middleline(line);
  else
  browser->goto_index(line);
  browser->make_item_visible();
}

void swap_cb(fltk::Widget *o, void *) 
{
  int a = -1, b = -1;
  for ( int t=0; t<browser->size(); t++ ) { // find two selected items
    if ( browser->selected(t) ) 
    {
      if ( a < 0 )
      { a = t; }
      else 
      { b = t; break; }
    }
  }
#warning no swap(int, int) -rs
//  browser->swap(a, b);        // swap them
/*
  if b > 0 swap is possible
  insert item-a after b   // line numbers change above b
  insert item-b after a   // line numbers change above a
  delete item-a           // line numbers above a restored
  delete item-b           // line numbers above b restored
*/
}

void cd_dirname(char* fpath)
{
  char buf[1024];
  strcpy(buf, fpath);
  char* p = strrchr(buf, '/');
  if (p)  { *p = 0; chdir(buf); }
}

int main(int argc, char **argv) 
{
  cd_dirname(argv[0]);
  int i;
  if (! /* translating to v 2.x, best guess is 'Fl::' = */ fltk::args(argc,argv,i))  /* translating to v 2.x, best guess is 'Fl::' = */ fltk::fatal( /* translating to v 2.x, best guess is 'Fl::' = */ fltk::help);
  const char* fname = (i < argc) ? argv[i] : "browser2.cxx";
  fltk::Window window(400,400,fname);
  window.begin();
  browser = new fltk::Browser(0,0,400,350,0);
//  browser->begin();
  browser->type(fltk::Browser::MULTI);
//browser->type(FL_HOLD_BROWSER);
//browser->color(42);
  browser->callback(b_cb, (void*)0);
// browser->scrollbar_right();
//browser->has_scrollbar(Fl_Browser::BOTH_ALWAYS);
  if (!browser->load(fname)) 
  {
#ifdef _MSC_VER
    // if 'browser' was started from the VisualC environment in Win32, 
    // the current directory is set to the environment itself, 
    // so we need to correct the browser file path
    int done = 1;
    if ( i == argc ) 
    
    {
      fname = "../test/browser2.cxx";
      done = browser->load(fname);
    }
    if ( !done )
    
    {
      printf("Can't load %s, %s\n", fname, strerror(errno));
      exit(1);
    }
#else
    printf("Can't load %s, %s\n", fname, strerror(errno));
    exit(1);
#endif
  }
  //browser->position(0);
  browser->yposition(0);

  field = new fltk::IntInput(50, 350, 350, 25, "Line #:");
  field->callback(show_cb);

  top = new fltk::Button(0, 375, 80, 25, "Top");
  top->callback(show_cb);

  bottom = new fltk::Button(80, 375, 80, 25, "Bottom");
  bottom->callback(show_cb);

  middle = new fltk::Button(160, 375, 80, 25, "Middle");
  middle->callback(show_cb);

  visible = new fltk::Button(240, 375, 80, 25, "Make Vis.");
  visible->callback(show_cb);

  swap = new fltk::Button(320, 375, 80, 25, "Swap");
  swap->callback(swap_cb);
  swap->tooltip("Swaps two selected lines\n(Use CTRL-click to select two lines)");

  window.resizable(browser);
  window.show(argc,argv);
  return fltk::run();
}

//
// End of "$Id: browser.cxx,v 1.5.2.6.2.7 2004/07/26 20:52:52 easysw Exp $".
//


